package jp.sf.amateras.builder;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class XMLValidator {
	
	private static SAXParserFactory factory = SAXParserFactory.newInstance();
	
	/** of[V */
	public static void doValidate(IResource resource){
		// XMLt@Cof[Vs
		if(resource instanceof IFile && resource.getName().endsWith(".xml")){
			IFile file = (IFile) resource;
			try {
				file.deleteMarkers(IMarker.PROBLEM, false, IResource.DEPTH_ZERO);
				XMLErrorHandler reporter = new XMLErrorHandler(file);
				SAXParser parser = factory.newSAXParser();
				parser.parse(file.getContents(), reporter);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}
	
	/** SAXp[Tɐݒ肷G[nh */
	private static class XMLErrorHandler extends DefaultHandler {
		
		private IFile file;

		public XMLErrorHandler(IFile file) {
			this.file = file;
		}

		private void addMarker(SAXParseException e, int severity) {
			String message = e.getMessage();
			int lineNumber = e.getLineNumber();
			try {
				IMarker marker = file.createMarker(IMarker.PROBLEM);
				marker.setAttribute(IMarker.MESSAGE, message);
				marker.setAttribute(IMarker.SEVERITY, severity);
				if (lineNumber == -1) {
					lineNumber = 1;
				}
				marker.setAttribute(IMarker.LINE_NUMBER, lineNumber);
			} catch (CoreException ex) {
				ex.printStackTrace();
			}
		}

		public void error(SAXParseException exception) throws SAXException {
			addMarker(exception, IMarker.SEVERITY_ERROR);
		}

		public void fatalError(SAXParseException exception) throws SAXException {
			addMarker(exception, IMarker.SEVERITY_ERROR);
		}

		public void warning(SAXParseException exception) throws SAXException {
			addMarker(exception, IMarker.SEVERITY_WARNING);
		}
	}	
}
